@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2023  Błażej Szczygieł
+    Copyright (C) 2010-2024  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -51,6 +51,9 @@
 #if defined(CHECK_FOR_EGL) || defined(Q_OS_MACOS)
     #include <QSurfaceFormat>
 #endif
+#ifdef Q_OS_ANDROID
+    #include <QtCore/private/qandroidextras_p.h>
+#endif
 
 #include <clocale>
 #include <csignal>
@@ -474,10 +477,6 @@ static QtMessageHandler g_defaultMsgHandler = nullptr;
 static QMutex g_messageHandlerMutex;
 static void messageHandler(QtMsgType type, const QMessageLogContext &context, const QString &message)
 {
-#ifdef USE_VULKAN
-    if (type == QtWarningMsg && context.category && qstrcmp(context.category, "default") == 0 && message.startsWith("failed to load vulkan", Qt::CaseInsensitive))
-        return;
-#endif
     bool qmplay2Log = false;
     if (QCoreApplication::instance())
     {
@@ -617,9 +616,6 @@ static void checkForEGL()
 }
 #endif
 
-#ifdef Q_OS_ANDROID
-Q_DECL_EXPORT
-#endif
 int main(int argc, char *argv[])
 {
     signal(SIGINT, signal_handler);
@@ -633,6 +629,10 @@ int main(int argc, char *argv[])
     signal(SIGTERM, signal_handler);
     atexit(exitProcedure);
 
+#ifdef Q_OS_ANDROID
+    qputenv("QT_ANDROID_ENABLE_RIGHT_MOUSE_FROM_LONG_PRESS", "1");
+    qputenv("QT_ANDROID_MINIMUM_MOUSE_DOUBLE_CLICK_DISTANCE", "1000"); // Otherwise double-tap doesn't work in Qt6
+#endif
     qputenv("QT_QPA_UPDATE_IDLE_TIME", "0");
 
 #ifdef Q_OS_HAIKU
@@ -641,6 +641,9 @@ int main(int argc, char *argv[])
 
 #ifdef Q_OS_ANDROID
     QGuiApplication::setAttribute(Qt::AA_DontUseNativeDialogs);
+    QGuiApplication::setAttribute(Qt::AA_DontUseNativeMenuBar); // Doesn't work in Qt6 (bug)
+    QtAndroidPrivate::requestPermission("android.permission.READ_EXTERNAL_STORAGE").result();
+    QtAndroidPrivate::requestPermission("android.permission.WRITE_EXTERNAL_STORAGE").result();
 #endif
 
     QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
