@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2023  Błażej Szczygieł
+    Copyright (C) 2010-2024  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -36,14 +36,14 @@
 #ifdef Q_OS_MACOS
     #include <QProcess>
 #endif
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     #include <QWinThumbnailToolButton>
     #include <QWinThumbnailToolBar>
     #include <QWinTaskbarProgress>
     #include <QWinTaskbarButton>
 #endif
-#ifdef Q_OS_ANDROID
-    #include <QtAndroid>
+#if defined(Q_OS_WIN) && QT_VERSION < QT_VERSION_CHECK(6, 4, 0)
+    #include <dwmapi.h>
 #endif
 #include <qevent.h>
 
@@ -447,13 +447,6 @@ MainWidget::MainWidget(QList<QPair<QString, QString>> &arguments)
         }
     }
 
-#ifdef Q_OS_ANDROID
-    QtAndroid::requestPermissionsSync({
-        "android.permission.READ_EXTERNAL_STORAGE",
-        "android.permission.WRITE_EXTERNAL_STORAGE",
-    });
-#endif
-
     playlistDock->load(QMPlay2Core.getSettingsDir() + "Playlist.pls");
 
     bool noplay = false;
@@ -686,7 +679,7 @@ void MainWidget::playStateChanged(bool b)
         menuBar->player->togglePlay->setIcon(QMPlay2Core.getIconFromTheme("media-playback-start"));
         menuBar->player->togglePlay->setText(tr("&Play"));
     }
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     if (m_taskBarProgress)
         m_taskBarProgress->setPaused(!b);
 #endif
@@ -1184,7 +1177,7 @@ void MainWidget::toggleFullScreen()
         dockWidgetState = saveState();
 #endif // Q_OS_ANDROID
 
-#if !defined Q_OS_MACOS && !defined Q_OS_ANDROID
+#if !defined Q_OS_MACOS
         menuBar->hide();
 #endif
         statusBar->hide();
@@ -1283,7 +1276,9 @@ void MainWidget::toggleFullScreen()
             if (QDockWidget *dw = QMPlay2Ext->getDockWidget())
                 dw->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
 
-#if !defined Q_OS_MACOS && !defined Q_OS_ANDROID
+#if defined(Q_OS_ANDROID)
+        menuBar->setVisible(true);
+#elif !defined Q_OS_MACOS
         menuBar->setVisible(!hideMenuAct->isChecked());
 #endif
         statusBar->show();
@@ -1430,7 +1425,7 @@ void MainWidget::browseSubsFile()
 void MainWidget::setSeekSMaximum(int max)
 {
     seekS->setMaximum(qMax(0, max * 10));
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     if (m_taskBarProgress)
     {
         m_taskBarProgress->setMaximum(seekS->maximum() / 10);
@@ -1467,7 +1462,7 @@ void MainWidget::updatePos(double pos)
             currPos = intPos;
         }
         seekS->setValue(pos * 10.0);
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         if (m_taskBarProgress)
             m_taskBarProgress->setValue(pos);
 #endif
@@ -1559,7 +1554,7 @@ void MainWidget::hideDocksSlot()
             hideDocks();
     }
 }
-void MainWidget::doRestoreState(const QByteArray &data)
+void MainWidget::doRestoreState(const QByteArray &data, bool doToggleCompactView)
 {
     if (isMaximized())
     {
@@ -1567,6 +1562,8 @@ void MainWidget::doRestoreState(const QByteArray &data)
         QTimer::singleShot(0, this, [=] {
             QTimer::singleShot(0, this, [=] {
                 restoreState(data);
+                if (doToggleCompactView)
+                    menuBar->window->toggleCompactView->trigger();
                 setUpdatesEnabled(true);
                 repaint();
             });
@@ -1575,6 +1572,8 @@ void MainWidget::doRestoreState(const QByteArray &data)
     else
     {
         restoreState(data);
+        if (doToggleCompactView)
+            menuBar->window->toggleCompactView->trigger();
     }
 }
 
@@ -1743,13 +1742,12 @@ void MainWidget::addChooseNextStreamKeyShortcuts()
     QMPlay2GUI.shortcutHandler->appendAction(nextSubsStream, "KeyBindings/NextSubsStream", "Ctrl+2");
 }
 
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 void MainWidget::setWindowsTaskBarFeatures()
 {
     if (QSysInfo::windowsVersion() < QSysInfo::WV_6_1)
         return;
 
-
     m_taskbarButton = new QWinTaskbarButton(this);
     m_taskbarButton->setWindow(windowHandle());
 
@@ -1810,6 +1808,19 @@ void MainWidget::setWindowsTaskBarFeatures()
     m_thumbnailToolBar->addButton(settings);
 }
 #endif
+#if defined(Q_OS_WIN) && QT_VERSION < QT_VERSION_CHECK(6, 4, 0)
+void MainWidget::setTitleBarStyle()
+{
+    if (auto winId = reinterpret_cast<HWND>(window()->internalWinId()))
+    {
+        constexpr WORD DwmwaUseImmersiveDarkMode = 20;
+        constexpr WORD DwmwaUseImmersiveDarkModeBefore20h1 = 19;
+        const BOOL darkBorder = (palette().window().color().lightnessF() < 0.5) ? TRUE : FALSE;
+        if (DwmSetWindowAttribute(winId, DwmwaUseImmersiveDarkMode, &darkBorder, sizeof(darkBorder)) != S_OK)
+            DwmSetWindowAttribute(winId, DwmwaUseImmersiveDarkModeBefore20h1, &darkBorder, sizeof(darkBorder));
+    }
+}
+#endif
 
 void MainWidget::keyPressEvent(QKeyEvent *e)
 {
@@ -2026,7 +2037,7 @@ void MainWidget::closeEvent(QCloseEvent *e)
 
     playC.stop(true);
 
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     m_taskBarProgress = nullptr;
     delete m_taskbarButton;
     delete m_thumbnailToolBar;
@@ -2048,12 +2059,13 @@ void MainWidget::showEvent(QShowEvent *)
         if (QMPlay2Core.getSettings().getBool("MainWidget/isMaximized"))
 #endif
             showMaximized();
-        doRestoreState(QMPlay2Core.getSettings().getByteArray("MainWidget/DockWidgetState"));
-#ifdef Q_OS_WIN
+        doRestoreState(QMPlay2Core.getSettings().getByteArray("MainWidget/DockWidgetState"), QMPlay2Core.getSettings().getBool("MainWidget/IsCompactView"));
+#if defined(Q_OS_WIN) && QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         setWindowsTaskBarFeatures();
 #endif
-        if (QMPlay2Core.getSettings().getBool("MainWidget/IsCompactView"))
-            menuBar->window->toggleCompactView->trigger();
+#if defined(Q_OS_WIN) && QT_VERSION < QT_VERSION_CHECK(6, 4, 0)
+        setTitleBarStyle();
+#endif
         wasShow = true;
     }
     menuBar->window->toggleVisibility->setText(tr("&Hide"));
@@ -2076,6 +2088,12 @@ void MainWidget::changeEvent(QEvent *e)
         if (isFullScreen() || isMaximized())
             videoDock->scheduleEnterEventWorkaround();
     }
+#if defined(Q_OS_WIN) && QT_VERSION < QT_VERSION_CHECK(6, 4, 0)
+    else if (e->type() == QEvent::PaletteChange)
+    {
+        setTitleBarStyle();
+    }
+#endif
     QMainWindow::changeEvent(e);
 }
 
