@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2023  Błażej Szczygieł
+    Copyright (C) 2010-2024  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -36,6 +36,7 @@
 #include <QStackedWidget>
 #include <QStandardPaths>
 #include <QStyleFactory>
+#include <QImageWriter>
 #include <QRadioButton>
 #include <QActionGroup>
 #include <QApplication>
@@ -57,6 +58,9 @@
 #include <QSpinBox>
 #include <QBuffer>
 #include <QLabel>
+#ifdef Q_OS_WIN
+#   include <QOperatingSystemVersion>
+#endif
 
 #include <KeyBindingsDialog.hpp>
 #include <Appearance.hpp>
@@ -304,6 +308,9 @@ SettingsWidget::SettingsWidget(int page, const QString &moduleName, QWidget *vid
         generalSettingsPage = new Ui::GeneralSettings;
         generalSettingsPage->setupUi(generalSettingsWidget);
 
+        for (const auto &fmt : QImageWriter::supportedImageFormats())
+            generalSettingsPage->screenshotFormatB->addItem("." + fmt);
+
         appendColon(generalSettingsPage->langL);
         appendColon(generalSettingsPage->styleL);
         appendColon(generalSettingsPage->encodingL);
@@ -846,16 +853,11 @@ void SettingsWidget::createRendererSettings()
         auto bypassCompositor = createBypassCompositor();
 
 #ifdef Q_OS_WIN
-        if (QSysInfo::windowsVersion() >= QSysInfo::WV_6_0)
-        {
-            bypassCompositor->setToolTip(tr("This can improve performance. Some video drivers can crash when enabled."));
-        }
-        else
-#endif
+        bypassCompositor->setToolTip(tr("This can improve performance. Some video drivers can crash when enabled."));
+#else
         if (QGuiApplication::platformName() != "xcb")
-        {
             bypassCompositor->setEnabled(false);
-        }
+#endif
 
         glOnWindow->setToolTip(tr(
             "Use QOpenGLWidget (render-to-texture), also enable OpenGL for visualizations. "
@@ -866,10 +868,7 @@ void SettingsWidget::createRendererSettings()
                 this, [=](bool checked) {
             vsync->setEnabled(!checked);
 #ifdef Q_OS_WIN
-            if (QSysInfo::windowsVersion() >= QSysInfo::WV_6_0)
-            {
-                bypassCompositor->setEnabled(!checked);
-            }
+            bypassCompositor->setEnabled(!checked);
 #endif
         });
 
@@ -954,7 +953,7 @@ void SettingsWidget::createRendererSettings()
 
 #ifdef Q_OS_WIN
             bypassCompositor->setEnabled(!noExclusiveFullScreenDevIDs->contains(devices->itemData(idx).toByteArray()));
-            hdr->setVisible(QSysInfo::windowsVersion() >= QSysInfo::WV_10_0);
+            hdr->setVisible(QOperatingSystemVersion::current() >= QOperatingSystemVersion::Windows10);
 #else
             hdr->setVisible(false);
 #endif
@@ -1010,16 +1009,11 @@ void SettingsWidget::createRendererSettings()
         ));
 
 #ifdef Q_OS_WIN
-        if (QSysInfo::windowsVersion() >= QSysInfo::WV_6_0)
-        {
-            bypassCompositor->setToolTip(tr("Allow for exclusive fullscreen. This can improve performance."));
-        }
-        else
-#endif
+        bypassCompositor->setToolTip(tr("Allow for exclusive fullscreen. This can improve performance."));
+#else
         if (QGuiApplication::platformName() != "xcb")
-        {
             bypassCompositor->setEnabled(false);
-        }
+#endif
 
         vsync->setCheckState(settings->getWithBounds("Vulkan/VSync", Qt::Unchecked, Qt::Checked));
         gpuDeint->setChecked(settings->getBool("Vulkan/AlwaysGPUDeint"));
