@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2023  Błażej Szczygieł
+    Copyright (C) 2010-2024  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -50,7 +50,7 @@ public: // Helpers
     static bool checkFiltersSupported(const shared_ptr<PhysicalDevice> &physicalDevice);
 
 public:
-    static shared_ptr<Instance> create();
+    static shared_ptr<Instance> create(bool doObtainPhysicalDevice);
 
 public:
     Instance(Priv);
@@ -59,7 +59,7 @@ public:
     void prepareDestroy() override;
 
 private:
-    void init();
+    void init(bool doObtainPhysicalDevice);
 
 public:
     QString name() const override;
@@ -71,6 +71,7 @@ public:
 
 public:
     inline QVulkanInstance *qVulkanInstance();
+    inline PFN_vkGetInstanceProcAddr getVkGetInstanceProcAddr();
 
     void obtainPhysicalDevice();
 
@@ -81,10 +82,16 @@ public:
     AVPixelFormats supportedPixelFormats() const;
 
     shared_ptr<Device> createDevice(const shared_ptr<PhysicalDevice> &physicalDevice);
+    shared_ptr<Device> createOrGetDevice(const shared_ptr<PhysicalDevice> &physicalDevice);
+
+    inline const vk::PhysicalDeviceFeatures2 &enabledDeviceFeatures() const;
 
     shared_ptr<BufferPool> createBufferPool();
     shared_ptr<ImagePool> createImagePool();
 
+    shared_ptr<function<void()>> setFiltersOnOtherQueueFamiliy();
+    inline bool hasFiltersOnOtherQueueFamiliy() const;
+
 private:
     bool isCompatibleDevice(const shared_ptr<PhysicalDevice> &physicalDevice) const override final;
     void sortPhysicalDevices(vector<shared_ptr<PhysicalDevice>> &physicalDevices) const override final;
@@ -93,15 +100,27 @@ private:
     void fillSupportedFormats();
 
 private:
-    static vk::PhysicalDeviceFeatures requiredPhysicalDeviceFeatures();
     static vector<const char *> requiredPhysicalDeviceExtenstions();
 
 private:
     QVulkanInstance *const m_qVulkanInstance;
+    PFN_vkGetInstanceProcAddr m_vkGetInstanceProcAddr = nullptr;
+
+    vk::UniqueDebugUtilsMessengerEXT m_debugUtilsMessanger;
+
+    vk::PhysicalDeviceSamplerYcbcrConversionFeatures m_sycf;
+    vk::PhysicalDeviceTimelineSemaphoreFeatures m_tsf;
+    vk::PhysicalDeviceSynchronization2Features m_s2f;
+    vk::PhysicalDeviceFeatures2 m_enabledDeviceFeatures;
 
     shared_ptr<PhysicalDevice> m_physicalDevice;
     AVPixelFormats m_supportedPixelFormats;
 
+    std::mutex m_createOrGetDeviceMutex;
+
+    function<void()> m_unsetFiltersOnOtherQueueFamiliyFn;
+    std::atomic<uint32_t> m_filtersOnOtherQueueFamiliy = {0};
+
     QWindow *m_testWin = nullptr;
 };
 
@@ -111,10 +130,24 @@ QVulkanInstance *Instance::qVulkanInstance()
 {
     return m_qVulkanInstance;
 }
+PFN_vkGetInstanceProcAddr Instance::getVkGetInstanceProcAddr()
+{
+     return m_vkGetInstanceProcAddr;
+}
 
 shared_ptr<PhysicalDevice> Instance::physicalDevice() const
 {
     return m_physicalDevice;
 }
 
+const vk::PhysicalDeviceFeatures2 &Instance::enabledDeviceFeatures() const
+{
+    return m_enabledDeviceFeatures;
+}
+
+bool Instance::hasFiltersOnOtherQueueFamiliy() const
+{
+    return (m_filtersOnOtherQueueFamiliy > 0);
+}
+
 }
